/* gdsii.c
 * last revised at 9, August, 1995
 *
 * for GDSII and PCstream Formats
 * Copyright (C) 1994-5, All rights reserved.
 * written by Gyudong Kim(chilly@eltn20.snu.ac.kr)
 * mainly imported from the work of Ook Kim(okim@cadvax.etri.re.kr)
 *
 */
#include "findhier.h"
#include "str.h"

extern FILE *dbfile;
extern struct branch *trees;
extern struct branch *subtop;
extern FORMAT Format;
extern char *(*mycase)();

static char buf[BUFSIZE];

#define read_file(A,B,C) fread(A,B,1,C)

static void skip_record(length,str)
int length;
FILE *str;
{
	register int i;

	if (length & 0x1) length ^= 0x1;
	for(i=0;i<length;i++) (void)fgetc(str);
}

static long a,temp,cnv;
static char *as,*temps,*cnvs;

void set_byte_order()
{
     unsigned int i;
     int j;

     i = sizeof(a);
     as = (char *)(&a);
     temps = (char *)(&temp);
     cnvs = (char *)(&cnv);

     a = 0;
     for(j=0;j<i;j++) {
          a <<= 8;
          a |= j;
     }
}

static int read_rec_h(length, read_type, str)
int *length;
int *read_type;
FILE *str;
{
	int j;

	if (!read_file((MEMT *)(&temp),4,str)) {
		/* End of file reached */
		return(0);
	}
	for(j=0;j<4;j++) cnvs[j]=temps[as[j]];
	*length = (int)((cnv >> 16) & 0xffff);
	*read_type = cnv & 0xffff;
	if (Format==PCSTR) {
		(*length) = BIG(*length);
	}
	return(1);
}

static int find_record(read_type,str)
int read_type;
FILE *str;
{
	int rec_type,count;

	for (;;) {
		if (!read_rec_h(&count,&rec_type,str)) return(0);
		if (rec_type == read_type) break;
		skip_record(count-4,str);
	}
	return(count);
}

static void anal_sref(pname,s_top)
char *pname;
struct branch **s_top;
{
	int count;
	char *iname;

	count=find_record(SNAME_T,dbfile);
	iname=(char *)myalloc((SIZE)(count-4),"anal_sref");
	(void)read_file((MEMT *)iname,count-4,dbfile);
	iname[count-4]=0;
	add_cell(iname,pname,s_top);
	(void)find_record(ENDEL_T,dbfile);
}

static int read_element(type_code,pname,s_top)
int type_code;
char *pname;
struct branch **s_top;
{
	switch(type_code) {
		case SREF_T:
		case AREF_T:
			anal_sref(pname,s_top);
			return(0);
	}
	return(1);
}

static void make_str(parent,s_top)
struct branch *parent,**s_top;
{
	int count,type;

	for (;read_rec_h(&count,&type,dbfile);) {
		if (type == ENDSTR_T) break;
		if (read_element(type,parent->name,s_top)) 
			skip_record(count-4,dbfile);
	}
	skip_record(count-4,dbfile);
}

static char *bgn_str_header()
{
	int count;

	if (!(count=find_record(BGNSTR_T,dbfile))) return((char *)NULL);
	skip_record(count-4,dbfile); /* skip time field */
	count=find_record(STRNAME_T,dbfile)-4;
	(void)read_file((MEMT *)buf,count,dbfile);
	buf[count]=0;
	return(buf);
}

void gds_init()
{
	for (;bgn_str_header()!=NULL;) create(buf);
	rewind(dbfile);
}

void gds_tree()
{
	char *tmp;
	struct branch *parent = NULL;

	for (;(tmp = bgn_str_header())!=NULL;) {
		subtop = (struct branch *)NULL;
		parent = (struct branch *)myalloc(sizeof(struct branch),"gds_tree");
		parent->name = global(tmp,"GDSTOP");
		parent->next = trees;
		trees = parent;
		make_str(parent,&subtop);
		parent->offspring = subtop;
	}
	(void)fclose(dbfile);
}

char *pcstream_read(str)
FILE *str;
{
	int count;
	static char iname[BUFSIZE];

	if (str == NULL) return((char *)NULL);
	if (!(count=find_record(SNAME_T,str))) return((char *)NULL);
	(void)read_file((MEMT *)iname,count-4,str);
	iname[count-4]=0;
	return((*mycase)(iname));
}
/* gdsii.c */
